class Board {
	// конструктор объекта доски
    constructor(width, height, mode, state) {
        this.mode = mode;
        this.width = width;
        this.height = height;
        this.state = state;
    }
    // настройка объекта доски
    setup() {
        for(let i = 0; i < this.height; i++ ) {
            $('#board').append(`<div class="row ${i}"></div>`)
        }
        
        for(let i = 0; i < this.height; i++) {
            let cells = '';
            for(let j = 0; j < this.width; j++) {
                cells += `<div class="cell ${j}" data-val="-1" data-coords="${j},${i}"></div>`
            }
            $(`#board .row.${i}`).append(cells);
        }
        // сделаем исходную карту доски
        this.setLayout({
            0: [],
            1: [
                [2,1], [3,1], [4,1], [6,1], [7,1], [8,1],
                [1,2], [5,2], [9,2],
                [1,3], [5,3], [9,3],
                [1,4], [5,4], [9,4],
                [2,5], [3,5], [4,5], [6,5], [7,5], [8,5],
            ],
            2: [[1,1], [5,1], [9,1],
                [1,5], [5,5], [9,5]],
        });
    }
    // обновить расположение ячеек
    setLayout(layout) {
        // сначала установим в ячейках стену (wall)
        $('.cell').each((i, cell) => {
            this.setValue($(cell), -1);
        });
        // установим в ячейках значения из параметра вызова
        const numStates = 10;
        for(let val = 1; val < numStates; val++) {
            if(layout[val]) {
                layout[val].forEach((coord) => {
                    this.setValue(element(coord), val);
                });
            }
        }
    }

    emptyLayout() { // создание пустого поля
        return {0: [], 1: [], 2: [], 3: [], 4: [], 5: [], 6:[], 7: [], 8: [], 9: []};
    }
    // 1 вариант поля
    allSquares() {
        const layout = this.emptyLayout();
        for(let x = 0; x < this.width; x++) {
            for(let y = 0; y < this.height; y++) {
                if(x === 0 && y!== 0) {
                    layout[4].push([x, y]);
                }
                else if(x === this.width-1) {
                    layout[2].push([x, y]);
                } else {
                    layout[1].push([x, y]);
                }
            }
        }
        return layout;
    }
    // 2 вариант поля
    roundabout() {
        const layout = this.emptyLayout();
        for(let x = 0; x < this.width; x++) {
            for(let y = 0; y < this.height; y++) {
                if(x === 0 || y === this.height-1) {
                    layout[4].push([x, y]);
                }
                else if(y === 0 ||  x === this.width-1) {
                    layout[9].push([x, y]);
                } else {
                    layout[1].push([x, y]);
                }
            }
        }
        return layout;
    }
    // 3 вариант поля
    halfandhalf() {
        const layout = this.emptyLayout();
        for(let x = 0; x < this.width; x++) {
            for(let y = 0; y < this.height; y++) {
                if(x > y) {
                    layout[3].push([x, y]);
                } else if(y > x) {
                    layout[2].push([x, y])
                } else if(x === y) {
                    layout[4].push([x, y])
                }
            }
        }
        return layout;
    }
    // получить вариант поля
    getLayout() {
        const layout = this.emptyLayout();
        $('#board .row .cell').each((i, cell) => {
            const val = getValue($(cell));
            if(val !== -1) {
                layout[val].push(getCoords($(cell)));
            }
        });
        console.log(JSON.stringify(layout));
        return layout;
    }
    // установить значение в ячейку
    setValue(element, val) {
        val = val > 8 ? 9 : val;
        val = val < -1 ? -1 : val;
    
        // сброс HTML и классов CSS
        element.removeClass("wall val0 val1 val2 val3 val4 val5 val6 val7 val8 val9");
        element.html('');

        // установка значения
        element.attr('data-val', val);
        // установка класса CSS
        if(val === -1) {
            // не добавляем HTML теги если уже стена
            element.addClass(`wall`);
            return;
        } else {
            element.addClass(`val${val}`);
        }
        // добавляем HTML теги
        if(this.mode === 'numbers') {
            element.html(val);
        } else {
            if(val == 0) {
                element.html('');
            } else if(val == 1) {
                element.html('&centerdot;');
            } else if(val == 2) {
                element.html('&bull;');
            } 
        }
    }
    // обновить счет
    updateCountScore(updateVal) {
        let score = parseInt($('#scores .count').html());
        score += parseInt(updateVal);
        $('#scores .count').html(score);
    }
    // обновить счетчик пройденного
    updateTraverseScore() {
        let score = parseInt($('#scores .traversed').html());
        score++;
        $('#scores .traversed').html(score);
    }
}
