class MoveRule {
    // Decider, Transformer, Action - функции имеют одинаковый формат:
    // [<function>, <string representation>] для удобства представления
    constructor(id, board, decider, transform, action) {
        this.id = id;
        this.board = board;
        this.decider = decider;
        this.transform = transform;
        this.action = action;
    }

    // возвращает состояние и выполняет действие
    execute() {
        if (this.decider[0](this.board.state.coords)) {
            this.transform[0](this.board.state.coords);
            this.action[0](this.board.state);
            return true;
        }
        return false;
    }
    // вывести значение в строку
    toString() {
        return `Если число ${this.decider[1]}, тогда ${this.transform[1]} и затем ${this.action[1]}`;
    }
}

class Decider {
    // конструктор решателя
    constructor(board) {
        this.board = board;
    }
    // если значение равно
    ifValIs(val) {
        return [() => getValue(element(this.board.state.coords)) == val, `равно ${val}`];
    }
    // если значение меньше
    ifValLessThan(val) {
        return [() => getValue(element(this.board.state.coords)) < val, `меньше ${val}`];
    }
    // если значение больше
    ifValGreaterThan(val) {
        return [() => getValue(element(this.board.state.coords)) > val, `больше ${val}`];
    }
}

class Transformer {
	// конструктор преобразователя
    constructor(board) {
        this.board = board;
    }
    // нечего делать
    doNothing() {
        return [() => null, `ничего не делать`];
    }
    // делаем значение
    make(val) {
        return [() => {
            const score = Math.abs(getValue(element(this.board.state.coords)) - val);
            this.board.updateCountScore(score);
            this.board.setValue(element(this.board.state.coords), val);
        }, `равен ${val}`]
    }
    // добавляем значение
    add(val) {
        return [() => {
            this.board.updateCountScore(val);
            this.board.setValue(element(this.board.state.coords), getValue(element(this.board.state.coords)) + val);
        }, `добавить ${val}`];
    }
    // вычитаем значение
    subtract(val) {
        return [() => {
            this.board.updateCountScore(val);
            this.board.setValue(element(this.board.state.coords), getValue(element(this.board.state.coords)) - val);
        }, `вычесть ${val}`];
    }
}

class Mover {
	// конструктор движителя
    constructor(board) {
        this.board = board;
    }
    // Эти методы можно записать в виде
    // {forward: {'LEFT': this.moveAbsLeft(), 'RIGHT': this.moveAbsRight()...}}
    // но здесь используем с оператором switch для удобства понимания
    goForward() {
        return [() => {
            switch (this.board.state.direction) {
                case 'LEFT':
                    this.moveAbsLeft();
                    break;
                case 'RIGHT':
                    this.moveAbsRight();
                    break;
                case 'UP':
                    this.moveAbsUp();
                    break;
                case 'DOWN':
                    this.moveAbsDown();
                    break;
            }
        }, 'идти вперед'];
    }

    goBack() {
        return [() => {
                switch (this.board.state.direction) {
                    case 'LEFT':
                        this.moveAbsRight();
                        break;
                    case 'RIGHT':
                        this.moveAbsLeft();
                        break;
                    case 'UP':
                        return this.moveAbsDown();
                        break;
                    case 'DOWN':
                        this.moveAbsUp();
                        break;
                }
            }, 'идти назад'];
    }

    goRight() {
        return [() => {
            switch (this.board.state.direction) {
                case 'LEFT':
                    this.moveAbsUp();
                    break;
                case 'RIGHT':
                    this.moveAbsDown();
                    break;
                case 'UP':
                    this.moveAbsRight();
                    break;
                case 'DOWN':
                    this.moveAbsLeft();
                    break;
            }
        }, 'идти направо'];
    }

    goLeft() {
        return [() => {
            switch (this.board.state.direction) {
                case 'LEFT':
                    this.moveAbsDown();
                    break;
                case 'RIGHT':
                    this.moveAbsUp();
                    break;
                case 'UP':
                    this.moveAbsLeft();
                    break;
                case 'DOWN':
                    this.moveAbsRight();
                    break;
            }
        }, 'идти налево'];
    }

    rotatePacman() {
		// повращаем колобка
        if($('.pacman').hasClass('pacman-UP')) {
            this.board.state.direction = 'RIGHT';
        } else if($('.pacman').hasClass('pacman-RIGHT')) {
            this.board.state.direction = 'DOWN';
        } else if($('.pacman').hasClass('pacman-DOWN')) {
            this.board.state.direction = 'LEFT';
        } else if($('.pacman').hasClass('pacman-LEFT')) {
            this.board.state.direction = 'UP';
        }
        // заменим старый класс картинки на новый
        $('.pacman').removeClass('pacman-UP pacman-DOWN pacman-LEFT pacman-RIGHT')
            .addClass(`pacman-${this.board.state.direction}`);
    }

    movePacman() {
		// обновляем ячейки движения
        $('.pacman').removeClass('pacman pacman-UP pacman-DOWN pacman-LEFT pacman-RIGHT');
        element(this.board.state.coords).addClass('pacman').addClass(`pacman-${this.board.state.direction}`);
        // обновляем счет
        this.board.updateTraverseScore();
    }

    isWall(coords) {
        return element(coords).hasClass('wall');
    }

    moveAbsUp() {
		//движение вверх
        this.board.state.direction = 'UP';
        if(this.board.state.coords[1] > 0 
            && !this.isWall([this.board.state.coords[0], this.board.state.coords[1] - 1])) {
            this.board.state.coords[1] -= 1;
        }
        this.movePacman(state);
    }

    moveAbsDown() {
		//движение вниз
        this.board.state.direction = 'DOWN';
        if(this.board.state.coords[1] < this.board.height - 1 
            && !this.isWall([this.board.state.coords[0], this.board.state.coords[1] + 1])) {
            this.board.state.coords[1] += 1;
        }
        this.movePacman(state);
    }

    moveAbsRight() {
		//движение вправо
        this.board.state.direction = 'RIGHT';
        if(this.board.state.coords[0] < this.board.width - 1 
            && !this.isWall([this.board.state.coords[0] + 1, this.board.state.coords[1]])) {
                this.board.state.coords[0] += 1;
        }
        this.movePacman(this.board.state);
    }

    moveAbsLeft() {
		//движение влево
        this.board.state.direction = 'LEFT';
        if(this.board.state.coords[0] > 0 
            && !this.isWall([this.board.state.coords[0] - 1, this.board.state.coords[1]])) {
                this.board.state.coords[0] -= 1;
        }
        this.movePacman(this.board.state);
    }
}
